package com.ruoyi.gysl.controller;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ZipUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.IoUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.docking.entity.SmartDeclaration;
import com.ruoyi.docking.service.SmartDeclarationService;
import com.ruoyi.gysl.entity.*;
import com.ruoyi.gysl.entity.request.AuditRequest;
import com.ruoyi.gysl.entity.request.BasicInformationPageReq;
import com.ruoyi.gysl.entity.response.BasicInformationResponse;
import com.ruoyi.gysl.entity.response.ProjectExcelInfo;
import com.ruoyi.gysl.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;


import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.Size;
import java.io.*;
import java.util.*;
import java.util.zip.ZipEntry;

import static java.lang.System.out;

/**
 * 基本信息(BasicInformation)表控制层
 *
 * @author makejava
 * @since 2025-02-24 09:25:51
 */
@Api(tags = "基本信息")
@RestController
@RequestMapping("/gysl/basicInformation")
public class BasicInformationController extends BaseController {

    // 1.审核通过之后再审核不要通知
    // 2.项目评价配置和详情那边同步显示
    // 3.换掉项目动态导出模板
    // 6.项目不填项目起止日期导入失败
    // 7.企业端统计项目总数有问题
    // 9.项目基础信息时间查询改了格式
    // 10.添加项目统计页面根据年份查找统计数据

    // 4.项目汇编管理修改pdf
    // 5.月度报告修改
    // 8.企业和政务登录


    /**
     * 基本信息
     */
    @Resource
    private BasicInformationService basicInformationService;

    /**
     * 规划信息
     */
    @Resource
    private PlanInformationService planInformationService;

    /**
     * 模型信息
     */
    @Resource
    private MxService mxService;

    /**
     * 消息通知
     */
    @Resource
    private SmartDeclarationService smartDeclarationService;

    /**
     * 五要素模型信息
     */
    @Resource
    private WysmxInformationService wysmxInformationService;


    /**
     * 企业入驻信息
     */
    @Resource
    private QyrzInformationService qyrzInformationService;

    /**
     * 其他信息
     */
    @Resource
    private ProjectOtherInfoService projectOtherInfoService;

    /**
     * 分页查询所有数据
     *
     * @param page                    分页对象
     * @param basicInformationPageReq 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "分页查询所有数据", response = BasicInformation.class)
    @GetMapping("/page")
    public AjaxResult selectAll(Page<BasicInformation> page, BasicInformationPageReq basicInformationPageReq) {
        // 获取当前登录用户是否是企业端
        try {
            if (SecurityUtils.hasRole("company") && !SecurityUtils.isAdmin(SecurityUtils.getUserId())) {
                basicInformationPageReq.setTyshxydm(SecurityUtils.getUsername());
            }
        } catch (Exception ignored) {
        }
        return success(basicInformationService.page(page, basicInformationPageReq));
    }

    /**
     * 企业端暂存
     */
    @PreAuthorize("@ss.hasAnyRoles('company')")
    @ApiOperation(value = "企业端暂存")
    @PostMapping("/staging")
    public AjaxResult staging(@RequestBody AuditRequest req) {
        BasicInformation byId = basicInformationService.getById(req.getBasicInformation().getId());
        if (byId.getStatus() != 1) {
            throw new ServiceException("不能暂存!");
        }
        return success(basicInformationService.staging(req));
    }

    /**
     * 企业端提交填报
     */
    @PreAuthorize("@ss.hasAnyRoles('company')")
    @ApiOperation(value = "企业端提交填报")
    @PostMapping("/fill")
    public AjaxResult fill(@RequestBody AuditRequest req) {
        BasicInformation byId = basicInformationService.getById(req.getBasicInformation().getId());
        if (byId.getStatus() != 1) {
            throw new ServiceException("暂不能提交!");
        }
        return success(basicInformationService.audit(req, 2, byId.getStatus()));
    }

    /**
     * 政务端审核
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,common')")
    @ApiOperation(value = "政务端审核")
    @PostMapping("/audit")
    public AjaxResult audit(@RequestBody AuditRequest req) {
        BasicInformation byId = basicInformationService.getById(req.getBasicInformation().getId());
        if (byId.getStatus() == 1) {
            throw new ServiceException("请等待企业填报!");
        }
        return success(basicInformationService.audit(req, 3, byId.getStatus()));
    }

    /**
     * 通过主键查询单条项目所有数据(企业端)
     *
     * @param id 主键
     * @return 单条数据
     */
    @PreAuthorize("@ss.hasAnyRoles('company')")
    @ApiOperation(value = "通过主键查询单条项目所有数据(企业端)", response = BasicInformationResponse.class)
    @GetMapping("/stagingInfo/{id}")
    public AjaxResult stagingInfo(@PathVariable Long id) {
        return success(basicInformationService.stagingInfo(id));
    }


    /**
     * 通过主键查询单条项目所有数据(政务端)
     *
     * @param id 主键
     * @return 单条数据
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,common')")
    @ApiOperation(value = "通过主键查询单条项目所有数据(政务端)", response = BasicInformationResponse.class)
    @GetMapping("/{id}")
    public AjaxResult selectOne(@PathVariable Long id) {
        return success(basicInformationService.selectOne(id));
    }


    /**
     * 删除数据
     *
     * @param idList 主键结合
     * @return 删除结果
     */
    @ApiOperation("删除数据")
    @DeleteMapping("/delete")
    public AjaxResult delete(@RequestParam("idList") List<Long> idList) {
        //要删除其余所有表对应的数据
        basicInformationService.removeByXmIds(idList);
        return success();
    }

    /**
     * 项目基本信息批量导入
     */
    @ApiOperation(value = "项目基本信息批量导入")
//    @PreAuthorize("@ss.hasAnyRoles('admin,common')")
    @PostMapping(value = "/import", consumes = "multipart/form-data")
    @Transactional(rollbackFor = Exception.class)
    public AjaxResult importTemplateProject(@RequestPart("file") MultipartFile file) throws Exception {
        ExcelUtil<BasicInformation> util = new ExcelUtil<>(BasicInformation.class);
        List<BasicInformation> proList = util.importExcel(file.getInputStream());
        StringBuilder successMsg = new StringBuilder();
        if (proList == null || proList.isEmpty()) {
            throw new ServiceException("项目导入数据不能为空");
        } else {
            List<Mx> list = mxService.list();
            basicInformationService.saveBatch(proList);
            proList.forEach(x -> {
                //添加项目规划信息
                PlanInformation planInformation = new PlanInformation();
                planInformation.setXmId(x.getId());
                planInformationService.save(planInformation);
                //添加五要素模型信息
                List<WysmxInformation> wys = new ArrayList<>();
                list.forEach(y -> {
                    WysmxInformation wysmxInformation = new WysmxInformation();
                    wysmxInformation.setYsmc(y.getName());
                    wysmxInformation.setZdname(y.getJtzb());
                    wysmxInformation.setXmId(x.getId());
                    wys.add(wysmxInformation);
                });
                wysmxInformationService.saveBatch(wys);
                //企业入驻信息
                QyrzInformation qyrzInformation = new QyrzInformation();
                qyrzInformation.setXmId(x.getId());
                qyrzInformationService.save(qyrzInformation);
                //基本信息
                String[] otherInfo = {"改造前容积率", "改造后容积率", "亩均税收(万元/每亩)", "亩均销售(万元/每亩)", "贷款情况", "荣誉情况"};
                List<ProjectOtherInfo> saveOther = new ArrayList<>();
                Arrays.asList(otherInfo).forEach(y -> {
                    ProjectOtherInfo projectOtherInfo = new ProjectOtherInfo();
                    projectOtherInfo.setXmId(x.getId());
                    projectOtherInfo.setZdname(y);
                    saveOther.add(projectOtherInfo);
                });
                projectOtherInfoService.saveBatch(saveOther);
                //新增消息(也就是智能提醒)
                SmartDeclaration notice = new SmartDeclaration();
                notice.setTyshxydm(x.getTyshxydm());
                notice.setContent("关于" + x.getName() + "项目,待填报的通知");
                smartDeclarationService.save(notice);
            });
            successMsg.append("导入成功");
        }
        return AjaxResult.success(successMsg);
    }

    /**
     * 导出项目数据
     */
//    @PreAuthorize("@ss.hasAnyRoles('admin,common')")
    @ApiOperation("导出项目数据")
    @PostMapping("/exportInfo")
    public void exportInfo(HttpServletResponse response) {
        List<ProjectExcelInfo> pE = basicInformationService.selectList();
        ExcelUtil<ProjectExcelInfo> util = new ExcelUtil<>(ProjectExcelInfo.class);
        util.exportExcel(response, pE, "项目数据");
    }


    /**
     * 模板信息
     */
    @ApiOperation("导出基本信息模板")
    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response) {
        ExcelUtil<BasicInformation> util = new ExcelUtil<>(BasicInformation.class);
        util.importTemplateExcel(response, "项目数据");
    }

    /**
     * 根据条件导出基本信息
     */
    @ApiOperation(value = "根据条件导出基本信息")
    @PostMapping(value = "/export")
    public void export(HttpServletResponse response, @RequestParam Long xmid) throws Exception {
        List<BasicInformation> filteredList = new ArrayList<>();
        BasicInformation a1 = basicInformationService.getById(xmid);
        filteredList.add(a1);
        ExcelUtil<BasicInformation> util = new ExcelUtil<>(BasicInformation.class);
        util.exportExcel(response, filteredList, "基本信息");
    }

//    @ApiOperation(value = "123")

    /// /    @PreAuthorize("@ss.hasAnyRoles('admin,common')")
//    @PostMapping(value = "/import1", consumes = "multipart/form-data")
//    @Transactional(rollbackFor = Exception.class)
//    public AjaxResult import1(@RequestPart("file") MultipartFile file) throws Exception {
//        ExcelUtil<ProjectExcelInfo> util = new ExcelUtil<>(ProjectExcelInfo.class);
//        List<ProjectExcelInfo> proList = util.importExcel(file.getInputStream());
//        proList.forEach(x->{
//            // 1. 分割字符串
//            String[] dates = x.getA1().split("-");
//            if (dates.length != 2) {
//                throw new IllegalArgumentException("无效的日期范围格式");
//            }
//            BasicInformation basicInformation = new BasicInformation();
//            basicInformation.setName(x.getName());
//            basicInformation.setXmfrdwxz(x.getXmfrdwxz());
//            basicInformation.setNature(x.getNature());
//            basicInformation.setBegainTime(dates[0].replace(".","-").trim());
//            basicInformation.setEndTime(dates[1].replace(".","-").trim());
//            basicInformation.setXzfl(x.getXzfl());
//            basicInformation.setJhtze(x.getJhtze());
//            basicInformation.setProjectLeader(x.getProjectLeader());
//            basicInformation.setPhone(x.getPhone());
//            basicInformation.setJsjd(x.getJsjd());
//            basicInformation.setJsms(x.getJsms());
//            basicInformation.setPrioritize(x.getPrioritize());
//            basicInformation.setStatus(3);
//            PlanInformation planInformation = new PlanInformation();
//            planInformation.setZydmj(x.getZydmj());
//            planInformation.setRjl(x.getRjl());
//            planInformation.setZjzmj(x.getZjzmj());
//            planInformation.setBzcjzmj(x.getBzcjzmj());
//            basicInformationService.save(basicInformation);
//            planInformation.setXmId(basicInformation.getId());
//            planInformationService.save(planInformation);
//        });
//        return AjaxResult.success();
//    }
    @ApiOperation(value = "导出word")
    @PostMapping("/wordExport")
    public void wordExport(HttpServletResponse response, @RequestBody List<Long> ids) throws Exception {
        // 创建临时 ZIP 文件
        java.io.File tempZipFile = java.io.File.createTempFile("archive", ".zip");

        // 设置文件
        try {
            List<File> fileList = CollUtil.newArrayList();
            for (long id : ids) {
                BasicInformationResponse a1 = basicInformationService.selectOne(id);
                Map<String, Object> map = new HashMap<>();
                map.put("name", a1.getBasicInformation().getName() != null ? a1.getBasicInformation().getName() : "/");
                map.put("ssgnq", a1.getBasicInformation().getSsgnq() != null ? a1.getBasicInformation().getSsgnq() : "/");
                map.put("jsdd", a1.getBasicInformation().getJsdd() != null ? a1.getBasicInformation().getJsdd() : "/");
                map.put("xmfrdwxz", a1.getBasicInformation().getXmfrdwxz() != null ? a1.getBasicInformation().getXmfrdwxz() : "/");
                map.put("tyshxydm", a1.getBasicInformation().getTyshxydm() != null ? a1.getBasicInformation().getTyshxydm() : "/");
                map.put("sgdw", a1.getBasicInformation().getSgdw() != null ? a1.getBasicInformation().getSgdw() : "/");
                map.put("sjdw", a1.getBasicInformation().getSjdw() != null ? a1.getBasicInformation().getSjdw() : "/");
                map.put("begainTime", a1.getBasicInformation().getBegainTime() != null ? a1.getBasicInformation().getBegainTime() : "/");
                map.put("endTime", a1.getBasicInformation().getEndTime() != null ? a1.getBasicInformation().getEndTime() : "/");
                map.put("prioritize",a1.getBasicInformation().getPrioritize() != null ? a1.getBasicInformation().getPrioritize() : "/");
                map.put("label",a1.getBasicInformation().getLabel() != null ? a1.getBasicInformation().getLabel() : "/");
                map.put("projectLeader", a1.getBasicInformation().getProjectLeader() != null ? a1.getBasicInformation().getProjectLeader() : "/");
                map.put("phone", a1.getBasicInformation().getPhone() != null ? a1.getBasicInformation().getPhone() : "/");
                map.put("zydmj", a1.getBasicInformation().getZydmj() != null ? a1.getBasicInformation().getZydmj() : "/");
                if(a1.getPlanInformation()!=null) {
                    map.put("rjl", a1.getPlanInformation().getRjl() != null ? a1.getPlanInformation().getRjl() : "/");
                    map.put("zjzmj", a1.getPlanInformation().getZjzmj() != null ? a1.getPlanInformation().getZjzmj() : "/");
                    map.put("jzds", a1.getPlanInformation().getJzds() != null ? a1.getPlanInformation().getJzds() : "/");
                    map.put("bzcjzmj",a1.getPlanInformation().getBzcjzmj() !=null ? a1.getPlanInformation().getBzcjzmj() :"/");
                    map.put("bzczmj", a1.getPlanInformation().getBzcjzmj() != null ? a1.getPlanInformation().getBzcjzmj() : "/");
                    map.put("jzmd", a1.getPlanInformation().getJzmd() != null ? a1.getPlanInformation().getJzmd() : "/");
                    map.put("zgjzcs", a1.getPlanInformation().getZgjzcs() != null ? a1.getPlanInformation().getZgjzcs() : "/");
                    map.put("zgjzgd", a1.getPlanInformation().getZgjzgd() != null ? a1.getPlanInformation().getZgjzgd() : "/");
                    map.put("fhdj", a1.getPlanInformation().getFhdj() != null ? a1.getPlanInformation().getFhdj() : "/");
                    map.put("jdctcw", a1.getPlanInformation().getJdctcw() != null ? a1.getPlanInformation().getJdctcw() : "/");
                    map.put("fjdctcw", a1.getPlanInformation().getFjdctcw() != null ? a1.getPlanInformation().getFjdctcw() : "/");
                }
                else{
                    map.put("rjl","/");
                    map.put("zjzmj","/");
                    map.put("jzds","/");
                    map.put("bzcjzmj","/");
                    map.put("bzczmj","/");
                    map.put("jzmd","/");
                    map.put("zgjzcs","/");
                    map.put("zgjzgd","/");
                    map.put("fhdj","/");
                    map.put("jdctcw","/");
                    map.put("fjdctcw","/");
                }
                if (a1.getBasicInformation().getNature() != null) {
                    switch (a1.getBasicInformation().getNature()) {
                        case 1:
                            map.put("nature", "国企");
                            break;
                        case 2:
                            map.put("nature", "外资企业");
                            break;
                        case 3:
                            map.put("nature", "民营企业");
                            break;
                        case 4:
                            map.put("nature", "其他");
                            break;
                        default:
                            map.put("nature", "/");
                    }
                } else {
                    map.put("nature", "/");
                }
                map.put("jhtze", a1.getBasicInformation().getJhtze() != null ? a1.getBasicInformation().getJhtze() : "/");
                map.put("ztze", a1.getBasicInformation().getZtze() != null ? a1.getBasicInformation().getZtze() : "/");
                if (a1.getBasicInformation().getXzfl() != null) {
                    switch (a1.getBasicInformation().getXzfl()) {
                        case 1:
                            map.put("xzfl", "已建");
                            break;
                        case 2:
                            map.put("xzfl", "在建");
                            break;
                        case 3:
                            map.put("xzfl", "拟建");
                            break;
                        default:
                            map.put("xzfl", "/");
                    }
                } else {
                    map.put("xzfl", "/");
                }

                map.put("begainTime1", a1.getBasicInformation().getBegainTime() != null ? a1.getBasicInformation().getBegainTime() : "/");
                map.put("jsjd", (a1.getBasicInformation().getJsjd() != null)
                        ? a1.getBasicInformation().getJsjd()
                        : "/");
                if (a1.getQyrzInformation()!=null) {
                    map.put("rzqys", (a1.getQyrzInformation().getRzqys() != null) ? a1.getQyrzInformation().getRzqys() : "/");
                    map.put("rzqyhylx", a1.getQyrzInformation().getRzqyhylx() != null ? a1.getQyrzInformation().getRzqyhylx() : "/");
                    map.put("rysl", a1.getQyrzInformation().getRysl() != null ? a1.getQyrzInformation().getRysl() : "/");
                    map.put("rzl", a1.getQyrzInformation().getRzl() != null ? a1.getQyrzInformation().getRzl() : "/");
                    map.put("yczmj", a1.getQyrzInformation().getYczmj() != null ? a1.getQyrzInformation().getYczmj() : "/");
                    map.put("kzczmj",a1.getQyrzInformation().getKzczmj() != null ? a1.getQyrzInformation().getKzczmj() : "/");
                    map.put("gycfpjzj", a1.getQyrzInformation().getGycfpjzj() != null ? a1.getQyrzInformation().getGycfpjzj() : "/");
                }else{
                    map.put("rzqys","/");
                    map.put("rzqyhylx","/");
                    map.put("rysl","/");
                    map.put("rzl","/");
                    map.put("yczmj","/");
                    map.put("kzczmj","/");
                    map.put("gycfpjzj","/");
                }
                File wordFile = new File(a1.getBasicInformation().getName() + ".docx");
                try (InputStream inputStream = basicInformationService.generateDocument(map);
                     OutputStream outputStream = new FileOutputStream(a1.getBasicInformation().getName() + ".docx")) {
                    if (inputStream != null) {
                        byte[] buffer = new byte[4096];
                        int bytesRead;
                        while ((bytesRead = inputStream.read(buffer)) != -1) {
                            outputStream.write(buffer, 0, bytesRead);
                        }
                    }

                } catch (IOException e) {
                    e.printStackTrace();
                }
                fileList.add(wordFile);
            }
            // 将生成的 Word 文件添加到 ZIP 文件中
            ZipUtil.zip(tempZipFile, false, fileList.toArray(new File[fileList.size()]));
            // 设置响应头
            response.setContentType("application/x-msdownload");
            response.setHeader("Content-Disposition", "attachment; filename=archive.zip");

            // 将 ZIP 文件内容写入响应输出流
            try (InputStream zipInputStream = FileUtil.getInputStream(tempZipFile);
                 OutputStream responseOutputStream = response.getOutputStream()) {
                IoUtil.copy(zipInputStream, responseOutputStream);
            }
        }
        finally {
            // 删除临时 ZIP 文件
            if (tempZipFile.exists()) {
                FileUtil.del(tempZipFile);
            }
            // 清理生成的 Word 文件
            for (long id : ids) {
                BasicInformationResponse a1 = basicInformationService.selectOne(id);
                File wordFile = new File(a1.getBasicInformation().getName() + ".docx");
                if (wordFile.exists()) {
                    FileUtil.del(wordFile);
                }
            }
        }
    }
}

